@@ -482,8 +482,7 @@ gpointer gps_thread(gpointer data) {
 }
 
 void gps_init(appdata_t *appdata) {
-  appdata->gps_state = malloc(sizeof(gps_state_t));
-  memset(appdata->gps_state, 0, sizeof(gps_state_t));
+  appdata->gps_state = g_new0(gps_state_t, 1);
 
   /* start a new thread to listen to gpsd */
   appdata->gps_state->mutex = g_mutex_new();
@@ -495,22 +494,52 @@ void gps_release(appdata_t *appdata) {
 #ifdef USE_MAEMO
   gpsbt_stop(&appdata->gps_state->context);
 #endif
-  free(appdata->gps_state);
+  g_free(appdata->gps_state);
 }
 
 #else
 
-#warning "liblocation interface not completely implemented"
-
 static void
 location_changed(LocationGPSDevice *device, gps_state_t *gps_state) {
 
-  gps_state->fix = 
-    (device->fix->fields & LOCATION_GPS_DEVICE_LATLONG_SET);
-  
-  if(gps_state->fix) { 
+  gps_state->fields = device->fix->fields;
+
+  if(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET) { 
     gps_state->latitude = device->fix->latitude;
     gps_state->longitude = device->fix->longitude;
+    gps_state->epe = device->fix->eph;
+  }
+
+  if(gps_state->fields & LOCATION_GPS_DEVICE_TRACK_SET)
+    gps_state->heading = device->fix->track;
+  
+  /* update list of sattelites */
+
+  /* free old list */
+  if(gps_state->sats.num) {
+    g_free(gps_state->sats.PRN);
+    g_free(gps_state->sats.used);
+    g_free(gps_state->sats.ss);
+    gps_state->sats.num = 0;
+  }
+
+  /* build new one */
+  if(device->satellites_in_view) {
+    gps_state->sats.PRN  = g_new0(int, device->satellites_in_view);
+    gps_state->sats.used = g_new0(int, device->satellites_in_view);
+    gps_state->sats.ss   = g_new0(int, device->satellites_in_view);
+
+    int i;
+    for(i=0;i<device->satellites_in_view;i++) {
+      LocationGPSDeviceSatellite *sat = 
+	g_ptr_array_index(device->satellites, i);
+
+      gps_state->sats.PRN[i]  = sat->prn;
+      gps_state->sats.used[i] = sat->in_use;
+      gps_state->sats.ss[i]   = sat->signal_strength;
+    }
+
+    gps_state->sats.num  = device->satellites_in_view;
   }
 }
 
@@ -529,12 +558,27 @@ void gps_init(appdata_t *appdata) {
     g_signal_connect(gps_state->device, "changed", 
 		     G_CALLBACK(location_changed), gps_state);
 
+#if MAEMO_VERSION_MAJOR < 5
+  gps_state->control = location_gpsd_control_get_default();
+
+  if(gps_state->control && gps_state->control->can_control) {
+    printf("Having control over GPSD and GPS is to be enabled, starting it\n");
+    location_gpsd_control_start(gps_state->control);
+  }
+#endif
 }
 
 void gps_release(appdata_t *appdata) {
   gps_state_t *gps_state = appdata->gps_state;
   
   if(!gps_state->device) return;
+
+#if MAEMO_VERSION_MAJOR < 5
+  if(gps_state->control && gps_state->control->can_control) {
+    printf("Having control over GPSD, stopping it\n");
+    location_gpsd_control_stop(gps_state->control);
+  }
+#endif
   
   /* Disconnect signal */
   g_signal_handler_disconnect(gps_state->device, gps_state->idd_changed);
@@ -551,7 +595,7 @@ pos_t *gps_get_pos(appdata_t *appdata) {
 
   gps_state_t *gps_state = appdata->gps_state;
 
-  if(!gps_state->fix)
+  if(!(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET))
     return NULL;
 
   pos.lat = gps_state->latitude;
@@ -560,11 +604,42 @@ pos_t *gps_get_pos(appdata_t *appdata) {
   return &pos; 
 }
 
-float gps_get_heading(appdata_t *appdata) { return NAN; }
+float gps_get_heading(appdata_t *appdata) { 
+  gps_state_t *gps_state = appdata->gps_state;
+ 
+  if(!(gps_state->fields & LOCATION_GPS_DEVICE_TRACK_SET))
+    return NAN;
 
-float gps_get_epe(appdata_t *appdata) { return NAN; }
+  return gps_state->heading; 
+}
 
-gps_sat_t *gps_get_sats(appdata_t *appdata) { return NULL; }
+float gps_get_epe(appdata_t *appdata) { 
+  gps_state_t *gps_state = appdata->gps_state;
+ 
+  if(!(gps_state->fields & LOCATION_GPS_DEVICE_LATLONG_SET))
+    return NAN;
+
+  return gps_state->epe; 
+}
+
+gps_sat_t *gps_get_sats(appdata_t *appdata) { 
+  gps_sat_t *retval = NULL;
+  gps_state_t *gps_state = appdata->gps_state;
+
+  if(gps_state->sats.num) {
+    retval = g_new0(gps_sat_t, 1);
+    retval->num = gps_state->sats.num;
+  
+    retval->PRN  = g_memdup(gps_state->sats.PRN, 
+			    sizeof(int)*gps_state->sats.num);
+    retval->used = g_memdup(gps_state->sats.used, 
+			    sizeof(int)*gps_state->sats.num);
+    retval->ss   = g_memdup(gps_state->sats.ss, 
+			    sizeof(int)*gps_state->sats.num);
+  }
+
+  return retval;
+}
 
 
 #endif
