@@ -24,7 +24,16 @@
 #if MAEMO_VERSION_MAJOR < 5
 #define USE_BREAD_CRUMB_TRAIL
 #else
+// these are on maemo fremantle
+#include <hildon/hildon-button.h>
+#include <hildon/hildon-text-view.h>
+#include <hildon/hildon-window-stack.h>
+#define USE_HILDON_TEXT_VIEW
 #define USE_STACKABLE_WINDOW
+#define NO_COPY_N_PASTE
+#define MENU_GPXLIST   0
+#define MENU_CACHELIST 1
+#define MENU_CACHE     2
 #endif
 #endif
 
@@ -41,7 +50,6 @@
 
 #ifdef USE_MAEMO
 #include <hildon/hildon-program.h>
-#include <hildon/hildon-text-view.h>
 #include <hildon/hildon-file-chooser-dialog.h>
 #include <hildon/hildon-file-system-model.h>
 #include <hildon/hildon-bread-crumb-trail.h>
@@ -53,6 +61,8 @@
 #if MAEMO_VERSION_MAJOR >= 5
 #define USE_PANNABLE_AREA
 #include <hildon/hildon-pannable-area.h>
+/* panning a gtkhtml view currently doesn't work well */
+#define PANNABLE_HTML
 #endif
 #endif
 
@@ -95,7 +105,6 @@ typedef struct {
   pos_t gps;                  /* saved last gps position */
 
   char *image_path;           /* path to cache images in */
-  gboolean load_images;       /* load images and cache them */
 
   int search;                 /* "search in" items */
   char *search_str; 
@@ -129,10 +138,12 @@ typedef struct {
   GtkWidget *menu_remove, *menu_close;
 #endif
 
+#ifndef NO_COPY_N_PASTE
   /* do enable/disable the edit entries */
   GtkWidget *menu_cut, *menu_copy, *menu_paste;
   GtkClipboard *clipboard;     /* clipboard for copy/paste */
   GtkTextBuffer *active_buffer;
+#endif
 
   /* save result here to be able to "goto" it */
   pos_t geomath;
@@ -168,9 +179,11 @@ typedef struct {
   GtkWidget *bct;
   gpx_t *search_results;
 
+#ifdef USE_BREAD_CRUMB_TRAIL
   /* reference to enable/disable these while walking the crumb trail */
   GtkWidget *menu_import, *menu_export;
   GtkWidget *menu_search;
+#endif
 
   /* keep track of current "path" in the bread crumb trail */
   gpx_t *cur_gpx;
@@ -227,6 +240,9 @@ extern void main_after_settings_redraw(appdata_t *appdata, int flags);
 #include <stdarg.h>
 extern void errorf(const char *fmt, ...);
 extern gboolean on_window_key_press(GtkWidget *widget, 
-				    GdkEventKey *event, gpointer data);
+				    GdkEventKey *event, appdata_t *appdata);
+#ifdef USE_STACKABLE_WINDOW
+HildonAppMenu *menu_create(appdata_t *appdata, int mode);
+#endif
 
 #endif // GPXVIEW_H
