@@ -27,7 +27,7 @@ struct icon_data {
   { NULL, CACHE_CONT_MAX+1 },  /* ICON_CACHE_SIZE */
   { NULL, 9 },                 /* ICON_STARS      */
   { NULL, LOG_TYPE_MAX+1 },    /* ICON_LOG        */
-  { NULL, 8 },                 /* ICON_HEADING    */
+  { NULL, 9 },                 /* ICON_HEADING    */
   { NULL, WPT_SYM_MAX+1 },     /* ICON_WPT        */
   { NULL, 1 },                 /* ICON_TB         */
   { NULL, 8 },                 /* ICON_MISC       */
@@ -64,7 +64,7 @@ const char *log_icon_name[] = {
 
 /* ICON_HEADING    */
 const char *heading_icon_name[] = {
-  "n", "ne", "e", "se", "s", "sw", "w", "nw"
+  "n", "ne", "e", "se", "s", "sw", "w", "nw", "none"
 };
 
 /* ICON_WPT        */
@@ -210,6 +210,9 @@ GtkWidget *icon_get_widget(int type, int num) {
 }
 
 GdkPixbuf *icon_bearing(pos_t from, pos_t to) {
+  if(!from.lat && !from.lon)
+    return icon_get(ICON_HEADING, 8);
+
   int idx = (gpx_pos_get_bearing(from, to)+22.5)/45.0;
 
   /* make sure we stay in icon bounds */
