@@ -120,8 +120,8 @@ gpx_t *choose_file(appdata_t *appdata, gboolean whole_dir) {
 		 HELP_ID_IMPORT, appdata->osso_context);
 #endif
 #else
-  dialog = gtk_file_chooser_dialog_new (whole_dir?_("Open Directory"):
-					_("Open File"),
+  dialog = gtk_file_chooser_dialog_new (whole_dir?_("Import directory"):
+					_("Import file"),
 			GTK_WINDOW(appdata->window),
 			whole_dir?GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER :
 					GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -264,6 +264,11 @@ void cachelist_redraw(appdata_t *appdata) {
   // gtk_widget_show_all(appdata->vbox);
 #else
 
+  if(!appdata->cur_view) {
+    printf("cachelist redraw: no active view\n");
+    return;
+  }
+
   g_assert(!appdata->cur_cache);
   int redraw = 0;
   if(appdata->search_results)
@@ -274,7 +279,16 @@ void cachelist_redraw(appdata_t *appdata) {
   }
 
   if(redraw) {
-    gtk_container_remove(GTK_CONTAINER(appdata->vbox), appdata->cur_view);
+    GtkWidget *container = appdata->vbox;
+
+    printf("redraw %d\n", redraw);
+
+#ifdef USE_STACKABLE_WINDOW
+    HildonWindowStack *stack = hildon_window_stack_get_default();
+    container = hildon_window_stack_peek(stack);
+#endif
+
+    gtk_container_remove(GTK_CONTAINER(container), appdata->cur_view);
     switch(redraw) {
     case 1:
       appdata->cur_view = cachelist_create(appdata, 
@@ -286,8 +300,14 @@ void cachelist_redraw(appdata_t *appdata) {
       break;
     }
 
-    gtk_box_pack_start_defaults(GTK_BOX(appdata->vbox), appdata->cur_view);
-    gtk_widget_show_all(appdata->vbox);
+#ifdef USE_STACKABLE_WINDOW
+    if(container != appdata->vbox)
+      gtk_container_add(GTK_CONTAINER(container), appdata->cur_view); 
+    else
+#endif
+      gtk_box_pack_start_defaults(GTK_BOX(container), appdata->cur_view);
+    
+    gtk_widget_show_all(container);
   }
 #endif
 }
@@ -300,6 +320,9 @@ static gboolean cachelist_update(gpointer data) {
 #ifdef USE_MAEMO
   appdata_t *appdata = (appdata_t*)data;
 
+  if(appdata->cur_cache)
+    return TRUE;
+
   if(appdata->cachelist_disable_screensaver)
     if (osso_display_blanking_pause(appdata->osso_context) != OSSO_OK) 
       fprintf(stderr, "error with display blank\n");
@@ -312,6 +335,7 @@ static gboolean cachelist_update(gpointer data) {
 }
 
 static void cachelist_timer_reset(appdata_t *appdata) {
+  printf("cachelist timer reset\n");
   g_assert(appdata->cachelist_handler_id);
   gtk_timeout_remove(appdata->cachelist_handler_id);
   appdata->cachelist_handler_id = 
@@ -543,13 +567,39 @@ void cachelist_dialog(appdata_t *appdata, gpx_t *gpx) {
 }
 #else
 #ifdef USE_STACKABLE_WINDOW
+static void search_result_free(gpx_t *result);
+
+void on_cachelist_destroy(GtkWidget *widget, appdata_t *appdata) {
+  if(appdata->search_results) {
+    search_result_free(appdata->search_results);
+    appdata->search_results = NULL;
+  }
+  appdata->cur_gpx = NULL;
+
+  /* restore cur_view */
+  appdata->cur_view = g_object_get_data(G_OBJECT(widget), "cur_view");
+}
+
 void cachelist_dialog(appdata_t *appdata, gpx_t *gpx) {
   GtkWidget *window = hildon_stackable_window_new();
 
-  gtk_window_set_title(GTK_WINDOW(window), gpx->name);
+  /* store last "cur_view" in window */
+  g_object_set_data(G_OBJECT(window), "cur_view", appdata->cur_view);
 
-  gtk_container_add(GTK_CONTAINER(window), 
-		    cachelist_create(appdata, gpx, NULL));
+  appdata->cur_gpx = gpx;
+  char *title = g_strdup_printf("GPXView - %s", gpx->name);
+  gtk_window_set_title(GTK_WINDOW(window), title);
+  g_free(title);
+
+  appdata->cur_view = cachelist_create(appdata, gpx, NULL);
+  gtk_container_add(GTK_CONTAINER(window), appdata->cur_view);
+		    
+
+  hildon_window_set_app_menu(HILDON_WINDOW(window), 
+			     menu_create(appdata, MENU_CACHELIST));
+
+  g_signal_connect(G_OBJECT(window), "destroy", 
+		   G_CALLBACK(on_cachelist_destroy), appdata);
 
   gtk_widget_show_all(window);
 }
@@ -761,7 +811,14 @@ static void gpxlist_view_onRowActivated(GtkTreeView *treeview,
 	gpx_busy_dialog_destroy(dialog);
       }
 #ifndef USE_BREAD_CRUMB_TRAIL
-      cachelist_dialog(appdata, gpx);
+#ifdef USE_STACKABLE_WINDOW
+      if(!appdata->cur_gpx)
+#endif
+	cachelist_dialog(appdata, gpx);
+#ifdef USE_STACKABLE_WINDOW
+      else
+	printf("selected gpx, but cachelist window already present\n");
+#endif
 #else
       gtk_container_remove(GTK_CONTAINER(appdata->vbox), appdata->cur_view);
       appdata->cur_view = cachelist_create(appdata, gpx, NULL);
@@ -966,10 +1023,24 @@ cb_menu_about(GtkWidget *window, gpointer data) {
 
   gtk_about_dialog_set_name(about, "GPXView");
   gtk_about_dialog_set_version(about, VERSION);
-  gtk_about_dialog_set_copyright(about, 
-	 _("GPXView (c) 2008 by\n" 
-	 "Till Harbaum <till@harbaum.org>\n"
-	 "Geocaching.com: Tantil"));
+
+#ifdef ENABLE_LIBLOCATION
+  char *uses = "uses liblocation";
+#elif defined(ENABLE_GPSBT)
+  char *uses = "uses gpsbt and gpsd";
+#else
+  char *uses = "uses gpsd";
+#endif
+  
+  char *str = g_strdup_printf("%s\n\n(%s)", 
+			      _("GPXView (c) 2008 by\n" 
+				"Till Harbaum <till@harbaum.org>\n"
+				"Geocaching.com: Tantil"),
+			      _(uses)
+			      );
+
+  gtk_about_dialog_set_copyright(about, str);
+  g_free(str);
 
   gtk_about_dialog_set_website(about,
        _("http://www.harbaum.org/till/maemo"));
@@ -1007,21 +1078,6 @@ cb_menu_adddir(GtkWidget *window, gpointer data) {
   if(new) gpxlist_add(appdata, new);
 }
 
-static void 
-cb_menu_geomath(GtkWidget *window, gpointer data) {
-  geomath_dialog((appdata_t *)data);
-}
-
-static void 
-cb_menu_geotext(GtkWidget *window, gpointer data) {
-  geotext_dialog((appdata_t *)data);
-}
-
-static void 
-cb_menu_precpos(GtkWidget *window, gpointer data) {
-  precise_position((appdata_t *)data);
-}
-
 #ifndef USE_PANNABLE_AREA
 static void 
 cb_menu_close(GtkWidget *window, gpointer data) {
@@ -1340,17 +1396,23 @@ cb_menu_search(GtkWidget *window, gpointer data) {
     /* don't search if we are asked to search for nothing */
     if(((appdata->search & (SEARCH_ID|SEARCH_NAME|SEARCH_DESC|SEARCH_OWNER)) &&
 	strlen(p) > 0) || (appdata->search & SEARCH_FINDS)) {
-      gpx_t *found = NULL;
 
       printf("Search for %s (flags = %x)...\n", p, appdata->search);
 
 #ifndef USE_BREAD_CRUMB_TRAIL
-      found = search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
+      gpx_t *found =
+	search_do(appdata, appdata->gpx, p, appdata->search, FALSE);
 
       /* do search result dialog here ... */
       cachelist_dialog(appdata, found);
+#ifndef USE_STACKABLE_WINDOW
       search_result_free(found);
 #else
+      appdata->search_results = found;
+#endif
+#else
+      gpx_t *found = NULL;
+
       if(appdata->cur_gpx)
 	found = search_do(appdata, appdata->cur_gpx, p, appdata->search, TRUE);
       else
@@ -1380,6 +1442,7 @@ cb_menu_quit(GtkWidget *window, gpointer data) {
 }
 #endif
 
+#ifndef NO_COPY_N_PASTE
 static void 
 cb_menu_cut(GtkWidget *widget, gpointer data) {
   appdata_t *appdata = (appdata_t*)data;
@@ -1424,6 +1487,7 @@ cb_menu_paste(GtkWidget *widget, gpointer data) {
   } else
     printf("paste: ERROR, no active buffer\n");
 }
+#endif
 
 static void 
 cb_menu_export_log(GtkWidget *widget, gpointer data) {
@@ -1445,11 +1509,190 @@ cb_menu_export_garmin(GtkWidget *widget, gpointer data) {
   garmin_export(appdata);
 }
 
+static void 
+cb_menu_geomath(GtkWidget *window, gpointer data) {
+  geomath_dialog((appdata_t *)data);
+}
+
+static void 
+cb_menu_geotext(GtkWidget *window, gpointer data) {
+  geotext_dialog((appdata_t *)data);
+}
+
+static void 
+cb_menu_precpos(GtkWidget *window, gpointer data) {
+  precise_position((appdata_t *)data);
+}
+
+#ifdef USE_STACKABLE_WINDOW
+static void on_export_destroy(GtkWidget *widget, appdata_t *appdata) {
+  appdata->cur_view = g_object_get_data(G_OBJECT(widget), "cur_view");
+}
+
+void on_export_clicked(GtkButton *button, appdata_t *appdata) {
+  HildonStackableWindow *view_window;
+  GtkWidget *but;
+
+  view_window = HILDON_STACKABLE_WINDOW (hildon_stackable_window_new ());
+  gtk_window_set_title (GTK_WINDOW (view_window), "GPXView - Export");
+
+   /* store last "cur_view" in window */
+  g_object_set_data(G_OBJECT(view_window), "cur_view", appdata->cur_view);
+  appdata->cur_view = NULL;
+
+  g_signal_connect(G_OBJECT(view_window), "destroy", 
+		   G_CALLBACK(on_export_destroy), appdata);
+
+  GtkVBox *contents = GTK_VBOX(gtk_vbox_new (12, FALSE));
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Export to Maemo Mapper"),
+	    _("Save a Maemo Mapper POI file"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_export_mmpoi), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Export Field Notes"),
+	    _("Save a Garmin Field Notes file"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_export_log), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+  
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Export Garmin GPX"),
+	    _("Save a GPX file containing modified waypoints"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_export_garmin), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+
+  gtk_container_add (GTK_CONTAINER (view_window), GTK_WIDGET (contents));
+  gtk_widget_show_all (GTK_WIDGET (view_window));
+}
+
+static void on_tools_destroy(GtkWidget *widget, appdata_t *appdata) {
+  appdata->cur_view = g_object_get_data(G_OBJECT(widget), "cur_view");
+}
+
+void on_tools_clicked(GtkButton *button, appdata_t *appdata) {
+  HildonStackableWindow *view_window;
+  GtkWidget *but;
+
+  view_window = HILDON_STACKABLE_WINDOW (hildon_stackable_window_new ());
+  gtk_window_set_title (GTK_WINDOW (view_window), "GPXView - Tools");
+
+   /* store last "cur_view" in window */
+  g_object_set_data(G_OBJECT(view_window), "cur_view", appdata->cur_view);
+  appdata->cur_view = NULL;
+
+  g_signal_connect(G_OBJECT(view_window), "destroy", 
+		   G_CALLBACK(on_tools_destroy), appdata);
+
+  GtkVBox *contents = GTK_VBOX(gtk_vbox_new (12, FALSE));
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Geomath"),
+	    _("Geocoordinate calculation"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_geomath), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Geotext"),
+	    _("Text analysis, letter counting etc"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_geotext), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+  
+
+  but = hildon_button_new_with_text(
+	    HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH,
+	    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
+	    _("Precise Position"),
+	    _("Calculate a very precise GPS position"));
+  g_signal_connect(but, "clicked", 
+		   G_CALLBACK(cb_menu_precpos), appdata); 
+  gtk_box_pack_start(GTK_BOX(contents), but, FALSE, FALSE, 0);
+
+  gtk_container_add (GTK_CONTAINER (view_window), GTK_WIDGET (contents));
+  gtk_widget_show_all (GTK_WIDGET (view_window));
+}
+
+HildonAppMenu *menu_create(appdata_t *appdata, int mode) { 
+  GtkWidget *button;
+  HildonAppMenu *menu = HILDON_APP_MENU(hildon_app_menu_new());
+
+  /* ------- */
+  button = gtk_button_new_with_label(_("About"));
+  g_signal_connect_after(button, "clicked", 
+			 G_CALLBACK(cb_menu_about), appdata);
+  hildon_app_menu_append(menu, GTK_BUTTON(button));
+
+  button = gtk_button_new_with_label(_("Settings"));
+  g_signal_connect_after(button, "clicked", G_CALLBACK(cb_menu_settings), 
+			 appdata);
+  hildon_app_menu_append(menu, GTK_BUTTON(button));
+
+  if(mode == MENU_GPXLIST) {
+    button = gtk_button_new_with_label(_("Import file"));
+    g_signal_connect_after(button, "clicked", 
+			   G_CALLBACK(cb_menu_add), appdata);
+    hildon_app_menu_append(menu, GTK_BUTTON(button));
+
+    button = gtk_button_new_with_label(_("Import directory"));
+    g_signal_connect_after(button, "clicked", 
+			   G_CALLBACK(cb_menu_adddir), appdata);
+    hildon_app_menu_append(menu, GTK_BUTTON(button));
+  }
+
+  if(mode == MENU_GPXLIST) {
+    button = gtk_button_new_with_label(_("Export"));
+    g_signal_connect_after(button, "clicked", 
+			   G_CALLBACK(on_export_clicked), appdata);
+    hildon_app_menu_append(menu, GTK_BUTTON(button));
+
+    button = gtk_button_new_with_label(_("Search"));
+    g_signal_connect_after(button, "clicked", 
+			   G_CALLBACK(cb_menu_search), appdata);
+    hildon_app_menu_append(menu, GTK_BUTTON(button));
+  }
+
+  button = gtk_button_new_with_label(_("Tools"));
+  g_signal_connect_after(button, "clicked", 
+			 G_CALLBACK(on_tools_clicked), appdata);
+  hildon_app_menu_append(menu, GTK_BUTTON(button));
+
+#ifdef HILDON_HELP
+  button = gtk_button_new_with_label(_("Help"));
+  g_signal_connect_after(button, "clicked", 
+			 G_CALLBACK(cb_menu_help), appdata);
+  hildon_app_menu_append(menu, GTK_BUTTON(button));
+#endif
+
+  gtk_widget_show_all(GTK_WIDGET(menu));
+
+  return menu;
+}
+#else
+
 void menu_create(appdata_t *appdata) { 
   GtkWidget *menu, *item;  
   menu = gtk_menu_new();
 
-#ifdef USE_MAEMO
+#ifdef USE_BREAD_CRUMB_TRAIL
   appdata->menu_import = 
 #endif
   item = gtk_menu_item_new_with_label(_("Import"));
@@ -1457,11 +1700,11 @@ void menu_create(appdata_t *appdata) {
   GtkWidget *submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
-  item = gtk_menu_item_new_with_label( _("File...") );
+  item = gtk_menu_item_new_with_label( _("File") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_add), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Directory...") );
+  item = gtk_menu_item_new_with_label( _("Directory") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_adddir), appdata);
 
@@ -1479,7 +1722,7 @@ void menu_create(appdata_t *appdata) {
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_remove), appdata);
 #endif
 
-#ifdef USE_MAEMO
+#ifdef USE_BREAD_CRUMB_TRAIL
   appdata->menu_export = 
 #endif
     item = gtk_menu_item_new_with_label(_("Export"));
@@ -1488,31 +1731,32 @@ void menu_create(appdata_t *appdata) {
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
 #ifdef USE_MAEMO
-  item = gtk_menu_item_new_with_label( _("Maemo Mapper POI...") );
+  item = gtk_menu_item_new_with_label( _("Maemo Mapper POI") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_export_mmpoi), appdata);
 #endif
 
-  item = gtk_menu_item_new_with_label( _("Garmin Field Notes...") );
+  item = gtk_menu_item_new_with_label( _("Garmin Field Notes") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_export_log), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Garmin GPX...") );
+  item = gtk_menu_item_new_with_label( _("Garmin GPX") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_export_garmin), appdata);
 
-#ifdef USE_MAEMO
+#ifdef USE_BREAD_CRUMB_TRAIL
   appdata->menu_search = 
 #endif
-  item = gtk_menu_item_new_with_label( _("Search...") );
+  item = gtk_menu_item_new_with_label( _("Search") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_search), appdata);
 
   gtk_menu_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
-
+ 
+#ifndef NO_COPY_N_PASTE
   /* ----------- copy'n paste submenu ----------------- */
   appdata->clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
   gtk_clipboard_set_can_store(appdata->clipboard, NULL, 0);
@@ -1534,8 +1778,9 @@ void menu_create(appdata_t *appdata) {
   gtk_widget_set_sensitive(item, FALSE);
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_paste), appdata);
+#endif
 
-  item = gtk_menu_item_new_with_label( _("Settings...") );
+  item = gtk_menu_item_new_with_label( _("Settings") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_settings), 
 		   appdata);
@@ -1547,17 +1792,17 @@ void menu_create(appdata_t *appdata) {
   submenu = gtk_menu_new();
   gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
 
-  item = gtk_menu_item_new_with_label( _("Geomath...") );
+  item = gtk_menu_item_new_with_label( _("Geomath") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_geomath), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Geotext...") );
+  item = gtk_menu_item_new_with_label( _("Geotext") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_geotext), appdata);
 
-  item = gtk_menu_item_new_with_label( _("Precise Position...") );
+  item = gtk_menu_item_new_with_label( _("Precise Position") );
   gtk_menu_append(GTK_MENU_SHELL(submenu), item);
   g_signal_connect(item, "activate", 
 		   GTK_SIGNAL_FUNC(cb_menu_precpos), appdata);
@@ -1565,12 +1810,12 @@ void menu_create(appdata_t *appdata) {
   gtk_menu_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
 
 #if defined(USE_MAEMO) && defined(HILDON_HELP)
-  item = gtk_menu_item_new_with_label( _("Help...") );
+  item = gtk_menu_item_new_with_label( _("Help") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_help), appdata);
 #endif
 
-  item = gtk_menu_item_new_with_label( _("About...") );
+  item = gtk_menu_item_new_with_label( _("About") );
   gtk_menu_append(GTK_MENU_SHELL(menu), item);
   g_signal_connect(item, "activate", GTK_SIGNAL_FUNC(cb_menu_about), appdata);
 
@@ -1596,6 +1841,7 @@ void menu_create(appdata_t *appdata) {
   gtk_box_pack_start(GTK_BOX(appdata->vbox), menu_bar, 0, 0, 0);
 #endif
 }
+#endif
 
 /********************* end of menu **********************/
 
@@ -1642,8 +1888,7 @@ void on_window_destroy (GtkWidget *widget, gpointer data) {
 }
 
 gboolean on_window_key_press(GtkWidget *widget, 
-			 GdkEventKey *event, gpointer data) {
-  appdata_t *appdata = (appdata_t*)data;
+			 GdkEventKey *event, appdata_t *appdata) {
   int handled = FALSE;
 
   //  printf("key event %d\n", event->keyval);
@@ -1651,15 +1896,6 @@ gboolean on_window_key_press(GtkWidget *widget,
   switch(event->keyval) {
 #ifdef USE_MAEMO
 
-#if 0  /* don't do this as it interferes with gtkentry fields */
-  case HILDON_HARDKEY_SELECT:
-    /* remove as long as we haven't reached the main gpx list */
-    if(appdata->cur_gpx)
-      hildon_bread_crumb_trail_pop(HILDON_BREAD_CRUMB_TRAIL(appdata->bct));
-    handled = TRUE;
-    break;
-#endif
-
 #ifdef HILDON_HARDKEY_INCREASE
   case HILDON_HARDKEY_INCREASE:
     html_zoom(appdata, TRUE);
@@ -1844,6 +2080,13 @@ static void crumb_add(appdata_t *appdata, char *name, int level,
 #endif // USE_BREAD_CRUMB_TRAIL
 
 void main_after_settings_redraw(appdata_t *appdata, int flags) {
+  printf("main after settings redraw\n");
+
+  if(!appdata->cur_view) {
+    printf("no active view\n");
+    return;
+  }
+
 #ifndef USE_MAEMO
   // in non-maemo setup this can only affect the main screen as
   // the menu is blocked while a dialog is open. also the main
@@ -1879,7 +2122,14 @@ void main_after_settings_redraw(appdata_t *appdata, int flags) {
   }
 
   if(redraw) {
-    gtk_container_remove(GTK_CONTAINER(appdata->vbox), appdata->cur_view);
+    GtkWidget *container = appdata->vbox;
+
+#ifdef USE_STACKABLE_WINDOW
+    HildonWindowStack *stack = hildon_window_stack_get_default();
+    container = hildon_window_stack_peek(stack);
+#endif
+
+    gtk_container_remove(GTK_CONTAINER(container), appdata->cur_view);
     switch(redraw) {
     case 1:
       appdata->cur_view = cachelist_create(appdata, 
@@ -1894,10 +2144,16 @@ void main_after_settings_redraw(appdata_t *appdata, int flags) {
       break;
     }
 
-    gtk_box_pack_start_defaults(GTK_BOX(appdata->vbox), appdata->cur_view);
-    gtk_widget_show_all(appdata->vbox);
-  }
+#ifdef USE_STACKABLE_WINDOW
+    if(container != appdata->vbox)
+      gtk_container_add(GTK_CONTAINER(container), appdata->cur_view); 
+    else
 #endif
+      gtk_box_pack_start_defaults(GTK_BOX(container), appdata->cur_view);
+    
+    gtk_widget_show_all(container);
+  }
+#endif // USE_MAEMO
 }
 
 int main(int argc, char *argv[]) {
@@ -1954,7 +2210,10 @@ int main(int argc, char *argv[]) {
   gtk_window_set_default_size(GTK_WINDOW(appdata.window), 500, 300);
 #endif
 
+#if MAEMO_VERSION_MAJOR == 5
   gtk_window_set_title(GTK_WINDOW(appdata.window), "GPXView");
+#endif
+
   g_signal_connect(G_OBJECT(appdata.window), "destroy", 
 		   G_CALLBACK(on_window_destroy), &appdata);
 
@@ -1964,7 +2223,12 @@ int main(int argc, char *argv[]) {
   appdata.vbox = gtk_vbox_new(FALSE, 2);
   gtk_container_add(GTK_CONTAINER(appdata.window), appdata.vbox);
 
+#ifndef USE_STACKABLE_WINDOW
   menu_create(&appdata);
+#else
+  hildon_window_set_app_menu(HILDON_WINDOW(appdata.window), 
+			     menu_create(&appdata, MENU_GPXLIST));
+#endif
 
 #ifdef USE_BREAD_CRUMB_TRAIL
   appdata.bct = hildon_bread_crumb_trail_new();
